/*****************************************************************************
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: kmath.h,v 1.3 2000/04/28 14:14:16 kevin Exp $
**  $Log: kmath.h,v $
**  Revision 1.3  2000/04/28 14:14:16  kevin
**  *** empty log message ***
**
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/
/******************************************************************************
 *
 * FILE IDENTIFICATION
 *
 *	File Name:	NUMR.H
 *	Author:		Kevin Rosenberg
 *	Purpose:	Header file containing definitions for numerical app's
 *	Date Started:	Nov 84
 *
 * DESCRIPTION
 *
 * MODIFICATION LOG
 *
 *****************************************************************************/

#ifndef _H_kmath
#define _H_kmath

#include <stdio.h>
#include <math.h>

#define PI      3.14159265358979323846
#define HALFPI  1.57079632679489661923	/* PI divided by 2 */
#define QUARTPI 0.78539816339744830962	/* PI divided by 4 */
#define I_PI	0.31830988618379067154	/* Inverse of PI */
#define I_PID2	0.63661977236758134308	/* Inverse of PID2 */
 
#define TWOPI	6.28318530717958647692
#define SQRT2   1.414213562373095049

#define F_EPSILON	1.0E-6
#define D_EPSILON	1.0E-10

#define DEG_TO_RAD(x)	(x*(PI/180.))
#define RAD_TO_DEG(x)	(x*(180./PI))


#define ASSUMEDZERO  1E-10

/* codes for C data types */

#define DT_CHAR		1
#define DT_INT		2
#define DT_LONG		3
#define DT_FLOAT	4
#define DT_DOUBLE	5
#define DT_STRING  6

typedef char	 *CMTX_1D;
typedef CMTX_1D  *CMTX_2D;
typedef CMTX_2D  *CMTX_3D;

typedef int	 *IMTX_1D;
typedef IMTX_1D  *IMTX_2D;
typedef IMTX_2D  *IMTX_3D;

typedef float 	 *FMTX_1D;
typedef FMTX_1D  *FMTX_2D;
typedef FMTX_2D  *FMTX_3D;

typedef double   *DMTX_1D;
typedef DMTX_1D  *DMTX_2D;
typedef DMTX_2D  *DMTX_3D;

union elem_val_un {	/* holds an element value */
	char	  c;
	int	  i;
	long int  l;
	float	  f;
	double	  d;
};

typedef union elem_val_un  MTX_ELEM_VAL;


union elem_ptr_un {	/* holds a pointer to a 1D vector of any type */
	char	  *c;
	int	  *i;
	long int  *l;
	float	  *f;
	double	  *d;
};

typedef union elem_ptr_un  MTX_1D;
typedef MTX_1D		  *MTX_2D;
typedef MTX_2D		  *MTX_3D;

union mtx_val_ptr_un {			/* pointer to matrix values */
    MTX_1D  m1;
    MTX_2D  m2;
    MTX_3D  m3;
};

typedef union mtx_val_ptr_un  MTX_PTR;

struct matrix_st {
    unsigned int order;			/* order, or dimension, of matrix */
    unsigned int elemtype;		/* element type */
    unsigned int elemsize;		/* size of element in bytes */
    unsigned int nx, ny, nz;		/* size of matrix in each dimension */
    MTX_PTR val;			/* pointer to matrix values */
};

typedef struct matrix_st MTX;
typedef struct matrix_st *MTXP;

/* DEFINITION IDENTIFICATION
 *
 *	Definitions to access a matrix element from an matrix
 *

#define me1(mtx,x)\
  (mtx->elemtype == DT_FLOAT ? mtx->val.m1.f[x] :\
    (mtx->elemtype == DT_DOUBLE ? mtx->val.m1.d[x] :\
      (mtx->elemtype == DT_INT ? mtx->val.m1.i[x] :\
	(mtx->elemtype == DT_LONG ? mtx->val.m1.l[x] :\
	  (mtx->elemtype == DT_CHAR ? mtx->val.m1.c[x] :\
	     0\
	      )))))

#define me2(mtx,x,y)\
  (mtx->elemtype == DT_FLOAT ? mtx->val.m2[x].f[y] :\
    (mtx->elemtype == DT_DOUBLE ? mtx->val.m2[x].d[y] :\
      (mtx->elemtype == DT_INT ? mtx->val.m2[x].i[y] :\
	(mtx->elemtype == DT_LONG ? mtx->val.m2[x].l[y] :\
	  (mtx->elemtype == DT_CHAR ? mtx->val.m2[x].c[y] :\
	    0\
	      )))))


#define me3(mtx,x,y,z)\
  (mtx->elemtype == DT_FLOAT ? mtx->val.m3[x][y].f[z] :\
    (mtx->elemtype == DT_DOUBLE ? mtx->val.m3[x][y].d[z] :\
      (mtx->elemtype == DT_INT ? mtx->val.m3[x][y].i[z] :\
	(mtx->elemtype == DT_LONG ? mtx->val.m3[x][y].l[z] :\
	  (mtx->elemtype == DT_CHAR ? mtx->val.m3[x][y].c[z] :\
	    0\
	       )))))
*/


/* lnearest.c */
long int lnearest(double x);
/* mtx_disp.c */
void mtx_show(const MTX *mtx);
void mtx_prt(const MTX *mtx, FILE *fp);
int mtx_prt_elem(const MTX *mtx, FILE *fp, unsigned int x, unsigned int y, unsigned int z);
/* mtx_elem.c */
int mtx_get_elem(const MTX *mtx, MTX_ELEM_VAL *me, const int x, const int y, const int z);
int mtx_put_elem(MTX *mtx, const MTX_ELEM_VAL *me, unsigned int x, unsigned int y, unsigned int z);
/* mtx_inp.c */
int mtx_inp_elem(const char *prompt, MTX_ELEM_VAL *mev, const int dtype);
/* mtx_main.c */
MTX *mtx_init(const unsigned int order, const unsigned int elem_type, const unsigned int nx, const unsigned int ny, const unsigned int nz);
MTX *mtx_clr(MTX *mtx);
int mtx_free(MTX *mtx);
int mtx_elem_size(const int dt);
int mtx_check(const MTX *mtx, const char *func_name);
/* norm_ang.c */
double norm_ang(double theta);
/* simpson.c */
double simpson(const double xmin, const double xmax, const double *y, const int np);

#endif
