/*****************************************************************************
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: kstddef.h,v 1.4 2000/04/28 18:00:55 kevin Exp $
**  $Log: kstddef.h,v $
**  Revision 1.4  2000/04/28 18:00:55  kevin
**  remove unused files
**
**  Revision 1.3  2000/04/28 17:38:16  kevin
**  Removed unused files
**
**  Revision 1.2  2000/04/28 14:14:16  kevin
**  *** empty log message ***
**
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/
/******************************************************************************
 *
 * FILE IDENTIFICATION
 *
 *	File Name:	STDDEF.H
 *	Author:		Kevin Rosenberg
 *	Purpose:	Header file containing KRL standard C definitions
 *	Date Started:	Dec. 83
 *
 * DESCRIPTION
 *	Header file contains KRL standard C definitions
 *
 * MODIFICATION LOG
 *
 *****************************************************************************/

#ifndef STDDEF_H
#define STDDEF_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#undef SHELL

#define STR_MAX_LEN 255
#define STR_SIZE    STR_MAX_LEN+1

#ifndef bool
typedef signed int bool;		/* Boolean variable type */
#endif

typedef unsigned char string[STR_SIZE];

#define UNSIGNLONG unsigned long int
#define UNSIGNCHAR unsigned char

#define TRUE	1
#define FALSE	0
#define OK	TRUE
#define ERROR	FALSE
#define YES	TRUE
#define NO	FALSE
#define ON	TRUE
#define OFF	FALSE

/*----------------------------------------------------------------------*/

#define SHOW(var, fmt)  fprintf (stderr, "var = fmt\n", var)

/*----------------------------------------------------------------------*/

#define INTERNAL_FUNC  static
#define INTERNAL_VAR   static

/*----------------------------------------------------------------------*/

#define	NEWLINE	'\n'
#define	TAB	'\t'
#define EOS	'\0'
#define BLANK	' '
#define ETX	26	/* end of text signal for PC-DOS */

/*----------------------------------------------------------------------*/

#define MAXPATHNAME  65		/* max length of pathname + 1 */
#define MAXFILENAME  13		/* rootname(8) + '.'(1) + extension(3) + EOS */
#define MAXROOTNAME   8
#define MAXEXTNAME    3
#define MAXFULLNAME  MAXPATHNAME + MAXFILENAME

/*----------------------------------------------------------------------*/

#define ABS(x)    ((x) < 0 ? -(x) : (x))
#define SQR(x)    ((x) * (x))

#define MAX(a,b)  ((a) > (b) ? (a) : (b))
#define MIN(a,b)  ((a) <= (b) ? (a) : (b))
#define ISWAP(a,b) {int i; i = a; a = b; b = i;}

#define CLIP(n,lb,ub)	if (n < lb) n = lb; else if (n > ub) n = ub
#define FOREVER         for (;;)
#define STR_EQUAL(s1,s2) (strcmp (s1, s2) == 0)

/*----------------------------------------------------------------------*/

struct time_st {
	int hour, minute, second, hs;
};

struct date_st {
	int year, month, date, dow;
};

struct timedate_st {
    struct time_st t;
    struct date_st d;
};

typedef struct time_st TIME;
typedef struct date_st DATE;
typedef struct timedate_st TIMEDATE;

/*----------------------------------------------------------------------*/

#define ERR_WARNING	0
#define ERR_SEVERE	1
#define ERR_FATAL	2

/*----------------------------------------------------------------------*/

/* codes for open command */

#ifdef LATTICE
#define OPEN_RDONLY  0x8000		/* Lattice codes for binary i/o */
#define OPEN_WRONLY  0x8001
#define OPEN_RDWR    0x8002
#elif MICROSOFT
#define OPEN_RDONLY  O_RDONLY			/* other system use standard codes */
#define OPEN_WRONLY  O_WRONLY			/* for binary */
#define OPEN_RDWR    O_RDWR
#else
#define OPEN_RDONLY  0			/* other system use standard codes */
#define OPEN_WRONLY  1			/* for binary */
#define OPEN_RDWR    2
#endif

/*----------------------------------------------------------------------*/

/* codes for input (inp_*) routines */

#define INP_ERROR   -1
#define INP_NO      FALSE
#define INP_YES     TRUE
#define INP_NEITHER 'b'
#define INP_RETURN  'r'
#define INP_BREAK   0x8000

#ifndef O_BINARY
#define O_BINARY (0)
#endif

#ifndef S_IWRITE
#define S_IWRITE S_IWUSR
#endif

/* allocnum.c */
float *alloc_float(int n);
double *alloc_double(int n);
int *alloc_int(int n);
/* fexist.c */
int file_exists(const char *fname);
/* fmax.c */
double fmax(double a, double b);
/* get_yn.c */
int get_yn(const char *prompt);
/* getch.c */
int getch(void);
void ungetch(int c);
/* kbasename.c */
char *kbasename(const char *filename);
/* iclip.c */
int iclip(int n, int lb, int ub);
/* s_head.c */
char *str_skip_head(const char *str, const char *charlist);
/* s_lower.c */
char *str_lower(char *s);
/* s_save.c */
char *str_save(const char *s);
/* s_upper.c */
char *str_upper(char *str);
/* s_wtail.c */
char *str_wrm_tail(char *str);
/* sysalloc.c */
void *sys_alloc(const int nbytes, const char *name);
/* syserror.c */
void sys_error(int severity, const char *msg, ...);
void sys_verror(int severity, const char *msg, va_list arg);
void sys_error_level(int severity);
/* sysfopen.c */
FILE *sys_fopen(const char *filename, const char *mode, const char *progname);
/* sysfree.c */
void sys_free(void *ptr, const char *name);
/* timedate.c */
DATE *td_get_date(DATE *d);
TIME *td_get_time(TIME *t);
double td_current_sec(void);
double td_time_to_sec(TIME *t);
TIME *td_time_sub(const TIME *t1, const TIME *t2, TIME *tdiff);
TIME *td_time_add(const TIME *t1, const TIME *t2, TIME *tsum);
TIME *td_time_copy(TIME *to, const TIME *from);
TIME *td_time_norm(TIME *t);
void td_get_tmdt(TIMEDATE *td);
const char *td_str_tmdt(const TIMEDATE *td);
const char *td_str_time(const TIME *t);
const char *td_str_stime(const TIME *t);
const char *td_str_date(const DATE *d);
char *td_str_cdate(DATE *d);
char *td_month_name(int n);
char *td_day_name(int n);

#endif
